@@ -2,7 +2,7 @@
 from __future__ import annotations
 
 import logging
-import typing
+from typing import *
 from datetime import datetime
 
 import sqlalchemy
@@ -10,7 +10,7 @@ import sqlalchemy
 from bi_etl.components.row.row import Row
 from bi_etl.components.row.row_status import RowStatus
 
-if typing.TYPE_CHECKING:
+if TYPE_CHECKING:
     from bi_etl.components.table import Table
     from bi_etl.scheduler.task import ETLTask
 
@@ -42,18 +42,18 @@ class BulkLoader(object):
             file_compression: str = '',
             options: str = '',
             analyze_compression: str = None,
-    ):
+    ) -> int:
         raise NotImplementedError()
 
     def load_from_iterator(
             self,
-            iterator: typing.Iterator,
+            iterator: Iterator,
             table_object: Table,
             table_to_load: str = None,
             perform_rename: bool = False,
             progress_frequency: int = 10,
             analyze_compression: str = None,
-            parent_task: typing.Optional[ETLTask] = None,
+            parent_task: Optional[ETLTask] = None,
     ) -> int:
         raise NotImplementedError()
 
@@ -83,16 +83,7 @@ class BulkLoader(object):
         if temp_table_name is not None:
             old_name = table_object.qualified_table_name + '_old'
 
-            # DROP TABLE IF EXISTS {old_name}
-            try:
-                with Table(
-                        table_object.task,
-                        table_name=old_name,
-                        database=table_object.database,
-                ) as old_table_object:
-                    old_table_object.table.drop()
-            except Exception:
-                pass
+            table_object.database.drop_table_if_exists(old_name)
 
             with Table(
                     table_object.task,
@@ -107,7 +98,7 @@ class BulkLoader(object):
     def apply_updates(
             self,
             table_object: Table,
-            update_rows: typing.Sequence[Row]):
+            update_rows: Sequence[Row]):
         """
         NOT TESTED !
         """
