@@ -1,6 +1,7 @@
 """
 Created on Apr 2, 2015
 """
+import io
 import os
 from typing import *
 from datetime import datetime, time
@@ -17,7 +18,7 @@ __all__ = ['XLSXReader']
 
 class XLSXReader(ETLComponent):
     """
-    XLSXReader will read rows from an Microsoft Excel XLSX formatted sheet.
+    XLSXReader will read rows from a Microsoft Excel XLSX formatted workbook.
     
     Parameters
     ----------
@@ -66,11 +67,12 @@ class XLSXReader(ETLComponent):
         (inherited from ETLComponent)
         
     restkey: str
-        Column name to catch long rows (extra values).
-        
+        Column name to catch extra long rows (more columns than we have column names)
+        when reading values (extra values).
+
     restval: str
-        The value to put in columns that are in the column_names but 
-        not present in a given row (missing values).     
+        The value to put in columns that are in the column_names but
+        not present in a given row when reading (missing values).
     """
     def __init__(self,
                  task: Optional[ETLTask],
@@ -82,7 +84,7 @@ class XLSXReader(ETLComponent):
         if logical_name is None:
             try: 
                 logical_name = os.path.basename(self.file_name)
-            except AttributeError:
+            except (AttributeError, TypeError):
                 logical_name = str(self.file_name)
         
         # Don't pass kwargs up. They should be set here at the end
@@ -111,22 +113,21 @@ class XLSXReader(ETLComponent):
         return f"XLSXReader({self.logical_name})"
     
     @property
-    def header_row(self):
+    def header_row(self) -> int:
         """
-        int
-            The sheet row to read headers from. Default = 1.
+        The sheet row to read headers from. Default = 1.
         """
         return self.__header_row
 
     @header_row.setter
-    def header_row(self, value):
+    def header_row(self, value: int):
         self.__header_row = value
         
     @property
-    def start_row(self):
+    def start_row(self) -> int:
         """
-        int
-            The sheet row to start reading data from. Default = header_row + 1
+        The sheet row to start reading data from.
+        Default = header_row + 1
         """
         if self.__start_row is not None:
             return self.__start_row
@@ -134,7 +135,7 @@ class XLSXReader(ETLComponent):
             return self.header_row + 1
 
     @start_row.setter
-    def start_row(self, value):
+    def start_row(self, value: int):
         self.__start_row = value
 
     def has_workbook_init(self) -> bool:
@@ -143,15 +144,24 @@ class XLSXReader(ETLComponent):
     @property
     def workbook(self):
         if self._workbook is None:
-            self._workbook = load_workbook(filename=self.file_name, read_only=True)
+            # Work around for openpyxl close not working correctly
+            with open(self.file_name, "rb") as f:
+                in_mem_file = io.BytesIO(f.read())
+            self._workbook = load_workbook(filename=in_mem_file, read_only=True)
+            # Original open code if the openpyxl libary can properly close the file
+            # self._workbook = load_workbook(filename=self.file_name, read_only=True)
         return self._workbook
     
     def set_active_worksheet_by_name(self, sheet_name: str):
         self._active_worksheet = self.workbook[sheet_name]
         self._active_worksheet_name = sheet_name
         self._column_names = None
+        self._full_iteration_header = None
         
     def set_active_worksheet_by_number(self, sheet_number: int):
+        """
+        Change to an existing worksheet based on the sheet number.
+        """
         sheet_names = self.get_sheet_names()
         if len(sheet_names) >= (sheet_number + 1):
             sheet_name = sheet_names[sheet_number]
@@ -173,7 +183,7 @@ class XLSXReader(ETLComponent):
         return self.active_worksheet.title
     
     def get_sheet_names(self):
-        return self.workbook.get_sheet_names()
+        return self.workbook.sheetnames
     
     def get_sheet_by_name(self, name):
         """Returns a worksheet by its name.
@@ -248,7 +258,10 @@ class XLSXReader(ETLComponent):
         len_column_names = len(self.column_names)
         this_iteration_header = self.full_iteration_header
         for row in self.active_worksheet.iter_rows(min_row=self.start_row):
-            self.__active_row += 1
+            if len(row) > 0:
+                self.__active_row = row[0].row
+            else:
+                self.__active_row += 1
             row_values = XLSXReader._get_cell_values(row)           
             d = self.Row(data=row_values[:len_column_names], iteration_header=this_iteration_header)
             len_column_names = len(self.column_names)
